@@ -35,6 +35,14 @@ impl LuaState {
 		server
 	}
 
+	/// Returns whether this is the menu Lua state or not.
+	pub unsafe fn is_menu(&self) -> bool {
+		self.get_global(crate::lua_string!("MENU_DLL"));
+		let menu = self.get_boolean(-1);
+		self.pop();
+		menu
+	}
+
 	/// Returns the Lua string as a slice of bytes.
 	///
 	/// **WARNING:** This will CHANGE the type of the value at the given index to a string.
@@ -166,6 +174,16 @@ impl LuaState {
 		(LUA_SHARED.lua_pushnil)(*self)
 	}
 
+	#[inline(always)]
+	pub unsafe fn push_thread(&self) -> i32 {
+		(LUA_SHARED.lua_pushthread)(*self)
+	}
+
+	#[inline(always)]
+	pub unsafe fn to_thread(&self, index: i32) -> State {
+		(LUA_SHARED.lua_tothread)(*self, index)
+	}
+
 	#[inline(always)]
 	pub unsafe fn pcall(&self, nargs: i32, nresults: i32, errfunc: i32) -> i32 {
 		(LUA_SHARED.lua_pcall)(*self, nargs, nresults, errfunc)
@@ -177,18 +195,27 @@ impl LuaState {
 	pub unsafe fn pcall_ignore(&self, nargs: i32, nresults: i32) -> bool {
 		let res = self.pcall(nargs, nresults, 0);
 		if res == LUA_ERRRUN {
-			self.get_global(crate::lua_string!("ErrorNoHaltWithStack"));
-			if self.is_nil(-1) {
-				eprintln!("[ERROR] {:?}", self.get_string(-2));
-				self.pop_n(2);
-				return false;
-			}
-			self.push_value(-2);
-			self.call(1, 0);
+			crate::lua_stack_guard!(self => {
+				self.get_global(crate::lua_string!("ErrorNoHaltWithStack"));
+				if self.is_nil(-1) {
+					eprintln!("[ERROR] {:?}", self.get_string(-2));
+					self.pop();
+				} else {
+					#[cfg(debug_assertions)] {
+						self.push_string(&format!("[pcall_ignore] {}", self.get_string(-2).expect("Expected a string here")));
+					}
+					#[cfg(not(debug_assertions))] {
+						self.push_value(-2);
+					}
+
+					self.call(1, 0);
+				}
+			});
 			self.pop();
-			return false;
+			false
+		} else {
+			true
 		}
-		true
 	}
 
 	pub unsafe fn load_string(&self, src: LuaString) -> Result<(), LuaError> {
@@ -450,9 +477,25 @@ impl LuaState {
 		(LUA_SHARED.lua_touserdata)(*self, index)
 	}
 
+	#[inline(always)]
+	pub unsafe fn coroutine_new(&self) -> State {
+		(LUA_SHARED.lua_newthread)(*self)
+	}
+
+	#[inline(always)]
+	pub unsafe fn coroutine_yield(&self, nresults: i32) -> i32 {
+		(LUA_SHARED.lua_yield)(*self, nresults)
+	}
+
+	#[inline(always)]
+	pub unsafe fn coroutine_resume(&self, narg: i32) -> i32 {
+		(LUA_SHARED.lua_resume)(*self, narg)
+	}
+
 	/// Creates a new table in the registry with the given `name` as the key if it doesn't already exist, and pushes it onto the stack.
 	///
 	/// Returns if the metatable was already present in the registry.
+	#[inline(always)]
 	pub unsafe fn new_metatable(&self, name: LuaString) -> bool {
 		(LUA_SHARED.lual_newmetatable)(*self, name) == 0
 	}
@@ -480,12 +523,14 @@ impl LuaState {
 			self.push_value(-2);
 			self.set_metatable(-2);
 			self.remove(self.get_top() - 1);
+			self.remove(self.get_top() - 1);
 		}
 
 		ptr.write(data);
 		ptr
 	}
 
+	#[cold]
 	pub unsafe fn error<S: AsRef<str>>(&self, msg: S) -> ! {
 		self.push_string(msg.as_ref());
 		(LUA_SHARED.lua_error)(*self);
@@ -589,6 +634,7 @@ impl LuaState {
 impl std::ops::Deref for LuaState {
 	type Target = *mut std::ffi::c_void;
 
+	#[inline(always)]
 	fn deref(&self) -> &Self::Target {
 		&self.0
 	}
